----------------------------------------------------------------------------------
-- Addon:         ProGuide - Guide Handler UI
-- Author:        Claus Gade Pedersen - Satisfyme
-- File:          GuideHandler.lua
-- Description:   Handles all guides and guide data
----------------------------------------------------------------------------------
-- Addon version: 10
-- File Revision: 10
-- Last Edit:     2011-09-12T21:30:43Z
-- Last Edit By:  satisfyme
----------------------------------------------------------------------------------

-- Local variables
local countStep = 0
local class, race = "", ""
local db = {}
local GUI = {}
local sName, sTexture, sLevel = "", "", 0
local L = ProGuideLocals
local ActiveObj = false
local tempGuideHolder = {}
local OtherLangGuides = {}
local GV = ProGuide_GuideValidat

-- returns a table uniqe number based on the table size with all sub tables
local function NumOfSteps(guide)
  if not guide or type(guide) ~= "table" then return 0 end

  countStep = 0
  for part, info in ipairs(guide.Part) do
    for step, value in ipairs(info.Step) do
      countStep = countStep + 1
    end
  end

  return countStep
end

-- Class Check
local function isClass(check)
  class = UnitClassBase("player")

  if type(check) == "string" and check == class then
    return true
  elseif type(check) == "table" and tContains(check, class) then
    return true
  else
    return false
  end
end

-- Race Check
local function isRace(check)
  race = UnitRace("player")

  if type(check) == "string" and check == race then
    return true
  elseif type(check) == "table" and tContains(check, race) then
    return true
  else
    return false
  end
end

-- Activate Guide if possible
local function ActivateGuide(guidename)
  if ProGuide.db then
    db = ProGuide.db.char.Guides
    if ProGuide.Guides[guidename]["TotalSteps"] == db[guidename]["TotalSteps"] and ProGuide.Guides[guidename]["Lang"] == GetLocale() then
      ProGuide.Guides[guidename]["Active"] = true
    else
      ProGuide.Guides[guidename]["Active"] = false
    end
  else
    ProGuide.Guides[guidename]["Active"] = false
  end
end

-- Get Current Part of guide
local function GetActivePartStep(guide)
  for part, info in ipairs(guide.Part) do
    if not info.Done then 
      for step, value in ipairs(guide.Part[part].Step) do
        if not value.Done then
          return part, step
        end
      end
      -- cleanup part since all step was done but part was not flaged done
      guide.Part[part].Step[step].Done = true
    end
  end
  -- if we reach this, then the guide is done and we need to tell that to the handler
  return 0,0
end

-- add guide to character guide progress
local function AddGuideToCharGuideDB(guidename, guide)
  if not guidename or type(guidename) ~= "string" then return end
  if not guide or type(guide) ~= "table" then return end

  db = ProGuide.db.char.Guides
  -- only add/update if not exist or TotalSteps not the same
  if (not db[guidename]) or (not db[guidename]["TotalSteps"] or db[guidename]["TotalSteps"] ~= NumOfSteps(guide)) then
    -- reset guide
    db[guidename] = {}
    db[guidename]["TotalSteps"] = NumOfSteps(guide)
    for part, info in ipairs(guide.Part) do
      -- add part status
      if not db[guidename]["Part"] then db[guidename]["Part"] = {} end
      db[guidename]["Part"][part] = {}
      db[guidename]["Part"][part]["Done"] = false
      for step, value in ipairs(info.Step) do
        -- add step status
        if not db[guidename]["Part"][part]["Step"] then db[guidename]["Part"][part]["Step"] = {} end
        db[guidename]["Part"][part]["Step"][step] = {}
        db[guidename]["Part"][part]["Step"][step]["Done"] = false
      end
    end
    return true
  else
    return false
  end
end

-- Check the guide for race and class and remove part and step from the guide that we done need
local function CheckClassAndRace(guide)
  if not guide then return guide end
  for p=#guide.Part, 1, -1 do
    if guide.Part[p].Race and not isRace(guide.Part[p].Race) then
      tremove(guide.Part, p)
    elseif guide.Part[p].Class and not isClass(guide.Part[p].Class) then
      tremove(guide.Part, p)
    else
      for s=#guide.Part[p].Step, 1, -1 do
        if guide.Part[p].Step[s].Race and not isRace(guide.Part[p].Step[s].Race) then
          tremove(guide.Part[p].Step, s)
        elseif guide.Part[p].Step[s].Class and not isClass(guide.Part[p].Step[s].Class) then
          tremove(guide.Part[p].Step, s)
        end
      end
    end
  end
  return guide
end

-- Check Content
local function ValidateContent(guide, isPart, isStep)
  local required = 0
  if isStep then

  elseif isPart then
    print("There is", #guide, "parts")
  else
    for key,val in pairs(guide) do
      -- check if key is in the GuideValidate
      if not GV[key] then guide[key] = nil end

      if guide[key] then
      -- If key is Part then handle parts
        if key == "Part" then
          guide[key] = ValidateContent(val, true)

      -- check if key type matches the validator
        elseif type(guide[key]) ~= GV[key] or guide[key] == "Required" then
          guide[key] = nil
        end
      end

      -- check if it is a required field
      if guide[key] and tContains(GV.Required, key) then required = required + 1 end
    end

    -- if we dont have all the required fields then return as a failed guide
    if #GV.Required ~= required then return false end
  end
  return guide
end

-- Check guide and remove objectives we dont need
local function CheckGuide(guide)
  -- Validate content and remove not valid content
  guide = ValidateContent(guide)

  -- remove class and race objectives that done fit the player character
  guide = CheckClassAndRace(guide)

  return guide
end

-- Add Preloaded Guides
function ProGuide:AddPreloadedGuides()
  for _, info in ipairs(tempGuideHolder) do
    ProGuide:AddGuide(info.lang, info.faction, info.guidename, info.guide)
  end
  tempGuideHolder = nil
end

-- Add data to database
function ProGuide:AddData(lang, data)
  -- Check if valid data have been added
  if not lang or lang ~= GetLocale() then return end
  if not data or type(data) ~= "table" then return end

  -- Add new data to Data database.
  for group,table in pairs(data) do
    -- Create the groupe if not there
    if not ProGuide.Data[group] then ProGuide.Data[group] = {} end

    for key, value in pairs(table) do
      -- add values if not found
      if not ProGuide.Data[group][key] then
        ProGuide.Data[group][key] = value
      -- update values if mustupdate value is presented
      elseif value.mustupdate then
        ProGuide.Data[group][key] = value
      end
    end
  end
end

-- Add guide to database
function ProGuide:AddGuide(lang, fac, guidename, guide)
  -- If Proguide is not online then add the guide to a temp guide holder until it's ready for parsing
  if not ProGuide.IsOnline then
    tinsert(tempGuideHolder, {
      ["lang"] = lang,
      ["faction"] = fac,
      ["guidename"] = guidename,
      ["guide"] = guide,
    })
    return
  end

  -- Do requirement checks
  if not lang then return end
  if lang ~= GetLocale() then
    tinsert(OtherLangGuides,{
      ["lang"] = lang,
      ["faction"] = fac,
      ["guidename"] = guidename,
      ["guide"] = guide,
    })
    return
  end
  if not fac then fac = UnitFactionGroup("player") end
  if fac ~= UnitFactionGroup("player") then return end
  if not guidename then return end
  if not guide or type(guide) ~= "table" then return end

  -- Check if player is the correct class if set
  if guide.Class and (not isClass(guide.Class)) then return end
  -- Check if player is the correct race if set
  if guide.Race and (not isRace(guide.Race)) then return end

  -- Check if there is one with the same name and then skip it so we don't overwrite it
  if ProGuide.Guides[guidename] then return end

  -----------------------------------------------------------------------
  -- Now we are ready to parse the guide
  -----------------------------------------------------------------------

  -- Check guide and remove objectives we dont need
  guide = CheckGuide(guide)
  if not guide then
    print(guidename .. L[": Failed guide validater!"])
    print(guidename .. L[" was not loaded."])
    return
  else
    print(guidename .. ": Is validated!")
  end

  -- Add the guide to Pro Guide
  --[[ProGuide.Guides[guidename] = guide
  ProGuide.Guides[guidename]["TotalSteps"] = NumOfSteps(guide)
  ProGuide.Guides[guidename]["Lang"] = lang

  -- Manage DB info
  AddGuideToCharGuideDB(guidename, guide)

  -- Activate guide if all is ready
  ActivateGuide(guidename) ]]
end

-- Load a guide into the Guide Frame
function ProGuide:LoadGuide(Obj, GuideName)
  if ProGuide.Guides[GuideName] and ProGuide.Guides[GuideName]["Active"] then
    db = ProGuide.db.char.Guides[GuideName]
    
    -- get what part and step we are at
    Obj.Part, Obj.Step = GetActivePartStep(db)
    
    -- if guide is done, then we will not load this guide
    if Obj.Part == 0 then return end
    
    -- Add guide name for later use
    Obj.GuideName = GuideName

    -- make shortcut for the guide
    GUI = ProGuide.Guides[GuideName]
    
    -- Add Icon if one is set
    if GUI.Icon then ProGuide:EditIcon(Obj, GUI.Icon) end
    
    -- Enable Buy List if set
    if GUI.Part[Obj.Part].UseBuyList then
      if not Obj.ButtonBuyList:IsEnabled() then Obj.ButtonBuyList:Enable() end
      Obj.BuyList.guide = GuideName
      Obj.BuyList.part = Obj.Part
    else
      if Obj.ButtonBuyList:IsEnabled() then Obj.ButtonBuyList:Disable() end
    end

    -- Add the Guide name to the objective frame
    ProGuide:EditHeadline(Obj, GUI.Name)
    
    -- Add current skill level if specified
    if GUI.ShowSkillLevel then ProGuide:SetSkillLevel(Obj, GUI.ShowSkillLevel) end

    -- Add Sub Title
    ProGuide:EditSubHeadline(Obj, GUI.Part[Obj.Part].Title)
    
    -- Add Objectives
    for step, info in ipairs(GUI.Part[Obj.Part].Step) do
      if step == Obj.Step then ActiveObj = true else ActiveObj = false end
      -- Add Train Skill to objective
      if info.TrainSkill then
        ProGuide:AddTrainSkill(Obj, ActiveObj, info.TrainSkill, db.Part[Obj.Part].Step[step].Done)
      end

      -- Add Learn Item to objective
      if info.LearnItem then
        ProGuide:AddLearnItem(Obj, ActiveObj, info.LearnItem, db.Part[Obj.Part].Step[step].Done)
      end

      -- Add Create to objective
      if info.Create then
        ProGuide:AddCreateItem(Obj, ActiveObj, info.Create, db.Part[Obj.Part].Step[step].Done)
      end

      -- Add Level Skill to objective
      if info.LevelSkill then
        ProGuide:AddLevelSkill(Obj, ActiveObj, info.LevelSkill, db.Part[Obj.Part].Step[step].Done)
      end

      --print(step)
    end
  end
end